# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTime v1.8.0

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.3:
* The `BasicZoneManager` and `ExtendedZoneManager` classes were unified under a
  new parent interface `ZoneManager`. This seems to have caused the flash size
  to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
  about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
  and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most flash
  size increase proportional to their limited 32kB limit.
* Adding the `ZoneManager` interface simplifies a lot of the complexity with
  saving and restoring time zones using the `TimeZoneData` object, and I think
  it is worth the extra cost of flash size. The mitigating factor is that
  applications targetted towards 8-bit processors will normally have fixed
  number of timezones at compile time, so they can avoid using a `ZoneManager`,
  and avoid this penalty in flash size.

In v1.4.1+:
* Removed the `ZoneInfo::transitionBufSize` field from the `ZoneInfo` struct,
  which saves 1 byte on 8-bit processors (none on 32-bit processors due to
  4-byte alignment). We save 266 bytes for `BasicZoneManager` and 386 bytes for
  `ExtendedZoneManager` when all the zones are loaded into the zone registry.
* Incorporated zoneName compression causes flash/ram usage to increase by
  ~250/120 bytes when using only 1-2 zones, but *decreases* flash consumption by
  1200-2400 bytes when all the zones are loaded into the `ZoneManager`.

In v1.5+:
* Changing `ZoneProcessorCache::getType()` from a `virtual` to a non-virtual
  method saves 250-350 bytes of flash memory when using a `BasicZoneManager` or
  an `ExtendedZoneManager` on an 8-bit AVR processor. Unexpectedly, the flash
  memory consumption *increases* slightly (~0-50 bytes) for some ARM processors
  and the ESP32. Since those processors have far more flash memory, this seems
  like a good tradeoff.
* Changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be subclasses of
  the templatized `BasicZoneProcessorTemplate` and
  `ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption
  by 250-400 bytes for 32-bit processors. Don't know why. (Very little
  difference for 8-bit AVR).
* Adding a `BrokerFactory` layer of indirection (to support more complex
  ZoneProcessors and Brokers) causes flash memory to go up by 100-200 bytes.

In v1.6:
* Added support for `LinkRegistry` to `BasicZoneManager` and
  `ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
  when using one of these classes due to the code required by `LinkRegistrar`.
  This extra cost is incurred even if the `LinkRegistry` is set to 0 elements.
  Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a
  `zonedb::kLinkRegistry` with 183 elements uses 1464 extra bytes of flash; a
  `zonedbx::kLinkRegistry` with 207 elements uses 1656 extra bytes.

In v1.7:
* The virtual destructor on the `Clock` base class removed. This reduced the
  flash usage by 618 bytes on AVR processors , 328 bytes on the SAMD21, but only
  50-60 bytes on other 32-bit processors.
* The various `printShortNameTo()` or `printShortTo()` methods changed to
  replace the underscore in the zone names (e.g. `Los_Angeles`) with spaces
  (e.g. `Los Angeles`) to be more human friendly. This made little difference in
  the flash memory consumption, except on the ESP32 where it increased by
  200-300 bytes.

In v1.7.2
* The `SystemClock::clockMillis()` is now non-virtual, using compile-time
  polymorphism through C++ template, and incorporating the same techniques from
  AceRoutine v1.3. Saves about 20-40 bytes of flash.

In v1.7.5:
* `ExtendedZoneProcessor.compareTransitionToMatch()` was modified to
  detect an exact equality between a `Transition` and its `MatchingEra` if any
  of the 3 time stamp versions ('w', 's', 'u') are equal. Adds about 120-150
  bytes of flash on 8-bit and 32-bit processors. But removing
  `originalTransitionTime` from `Transition` decreases flash usage by about 20
  bytes.
* Upgrade ESP8266 Core from 2.7.4 to 3.0.2. Flash consumption increases by
  3-5 kB across the board.
* Upgrade Teensyduino from 1.54 to 1.55. Add memory consumed by `malloc()` and
  `free()` when using classes with virtual methods into baseline
  MemoryBenchmark, reducing the actual memory usage of various features by
  ~3kB.

In v1.8.0:
* Move Clock and SystemClock benchmarks into AceTimeClock v1.0.0.
* Extract thin links from BasicZoneManager and ExtendedZoneManager into
  new BasicLinkManager and ExtendedLinkManager classes.
    * Saves 200-500 bytes of flash for BasicZoneManager and ExtendedZoneManager.
    * Applications can decide whether to use thin links through the LinkManager
      (~2000 flash bytes for AVR) or use fat links through the
      `kZoneAndLinkRegistry` (~5000 flash bytes for AVR).
* Create various test objects as global variables instead of stack variables
  to get a more accurate measurement of their static memory consumption.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Arduino AVR Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |    474/   11 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1128/   19 |   654/    8 |
| ZonedDateTime                          |   1378/   26 |   904/   15 |
| Manual ZoneManager                     |   1546/   13 |  1072/    2 |
| Basic TimeZone (1 zone)                |   6094/  315 |  5620/  304 |
| Basic TimeZone (2 zones)               |   6390/  435 |  5916/  424 |
| BasicZoneManager (1 zone)              |   7448/  341 |  6974/  330 |
| BasicZoneManager (all zones)           |  20200/  717 | 19726/  706 |
| BasicZoneManager (all zones+links)     |  24528/  717 | 24054/  706 |
| BasicLinkManager                       |   2446/   26 |  1972/   15 |
| Extended TimeZone (1 zone)             |   8976/  670 |  8502/  659 |
| Extended TimeZone (2 zones)            |   9368/ 1111 |  8894/ 1100 |
| ExtendedZoneManager (1 zone)           |  10358/  691 |  9884/  680 |
| ExtendedZoneManager (all zones)        |  32076/ 1175 | 31602/ 1164 |
| ExtendedZoneManager (all zones+links)  |  36956/ 1175 | 36482/ 1164 |
| ExtendedLinkManager                    |   2638/   26 |  2164/   15 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   3470/  153 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4100/  159 |   630/    6 |
| ZonedDateTime                          |   4350/  166 |   880/   13 |
| Manual ZoneManager                     |   4540/  153 |  1070/    0 |
| Basic TimeZone (1 zone)                |   9048/  453 |  5578/  300 |
| Basic TimeZone (2 zones)               |   9346/  575 |  5876/  422 |
| BasicZoneManager (1 zone)              |  10402/  479 |  6932/  326 |
| BasicZoneManager (all zones)           |  23172/  857 | 19702/  704 |
| BasicZoneManager (all zones+links)     |  27500/  857 | 24030/  704 |
| BasicLinkManager                       |   5420/  168 |  1950/   15 |
| Extended TimeZone (1 zone)             |  11930/  808 |  8460/  655 |
| Extended TimeZone (2 zones)            |  12324/ 1251 |  8854/ 1098 |
| ExtendedZoneManager (1 zone)           |  13312/  829 |  9842/  676 |
| ExtendedZoneManager (all zones)        |  35046/ 1313 | 31576/ 1160 |
| ExtendedZoneManager (all zones+links)  |  39926/ 1313 | 36456/ 1160 |
| ExtendedLinkManager                    |   5612/  168 |  2142/   15 |
+---------------------------------------------------------------------+

```

## SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Sparkfun SAMD Boards 1.8.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10064/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10576/    0 |   512/    0 |
| ZonedDateTime                          |  10664/    0 |   600/    0 |
| Manual ZoneManager                     |  10608/    0 |   544/    0 |
| Basic TimeZone (1 zone)                |  14556/    0 |  4492/    0 |
| Basic TimeZone (2 zones)               |  14836/    0 |  4772/    0 |
| BasicZoneManager (1 zone)              |  15316/    0 |  5252/    0 |
| BasicZoneManager (all zones)           |  32428/    0 | 22364/    0 |
| BasicZoneManager (all zones+links)     |  39028/    0 | 28964/    0 |
| BasicLinkManager                       |  11832/    0 |  1768/    0 |
| Extended TimeZone (1 zone)             |  16444/    0 |  6380/    0 |
| Extended TimeZone (2 zones)            |  16764/    0 |  6700/    0 |
| ExtendedZoneManager (1 zone)           |  17156/    0 |  7092/    0 |
| ExtendedZoneManager (all zones)        |  46740/    0 | 36676/    0 |
| ExtendedZoneManager (all zones+links)  |  54172/    0 | 44108/    0 |
| ExtendedLinkManager                    |  12024/    0 |  1960/    0 |
+---------------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* STM32duino 2.0.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  21420/ 3536 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  21696/ 3548 |   276/   12 |
| ZonedDateTime                          |  21776/ 3560 |   356/   24 |
| Manual ZoneManager                     |  21700/ 3540 |   280/    4 |
| Basic TimeZone (1 zone)                |  25548/ 3704 |  4128/  168 |
| Basic TimeZone (2 zones)               |  25852/ 3868 |  4432/  332 |
| BasicZoneManager (1 zone)              |  26288/ 3720 |  4868/  184 |
| BasicZoneManager (all zones)           |  43128/ 3720 | 21708/  184 |
| BasicZoneManager (all zones+links)     |  49576/ 3720 | 28156/  184 |
| BasicLinkManager                       |  23164/ 3548 |  1744/   12 |
| Extended TimeZone (1 zone)             |  27232/ 4092 |  5812/  556 |
| Extended TimeZone (2 zones)            |  27548/ 4644 |  6128/ 1108 |
| ExtendedZoneManager (1 zone)           |  27940/ 4096 |  6520/  560 |
| ExtendedZoneManager (all zones)        |  57120/ 4096 | 35700/  560 |
| ExtendedZoneManager (all zones+links)  |  64388/ 4096 | 42968/  560 |
| ExtendedLinkManager                    |  23356/ 3548 |  1936/   12 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 260089/27892 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 260545/27908 |   456/   16 |
| ZonedDateTime                          | 260641/27924 |   552/   32 |
| Manual ZoneManager                     | 260557/27896 |   468/    4 |
| Basic TimeZone (1 zone)                | 266169/28644 |  6080/  752 |
| Basic TimeZone (2 zones)               | 266537/28804 |  6448/  912 |
| BasicZoneManager (1 zone)              | 267225/28660 |  7136/  768 |
| BasicZoneManager (all zones)           | 284489/28660 | 24400/  768 |
| BasicZoneManager (all zones+links)     | 291193/28660 | 31104/  768 |
| BasicLinkManager                       | 261997/27912 |  1908/   20 |
| Extended TimeZone (1 zone)             | 268441/29172 |  8352/ 1280 |
| Extended TimeZone (2 zones)            | 268745/29724 |  8656/ 1832 |
| ExtendedZoneManager (1 zone)           | 269449/29180 |  9360/ 1288 |
| ExtendedZoneManager (all zones)        | 299261/29176 | 39172/ 1284 |
| ExtendedZoneManager (all zones+links)  | 306829/29176 | 46740/ 1284 |
| ExtendedLinkManager                    | 262189/27912 |  2100/   20 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP32 Boards 1.0.6

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 197748/13084 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 199560/13212 |  1812/  128 |
| ZonedDateTime                          | 199708/13228 |  1960/  144 |
| Manual ZoneManager                     | 199656/13204 |  1908/  120 |
| Basic TimeZone (1 zone)                | 204188/13372 |  6440/  288 |
| Basic TimeZone (2 zones)               | 204516/13532 |  6768/  448 |
| BasicZoneManager (1 zone)              | 205172/13388 |  7424/  304 |
| BasicZoneManager (all zones)           | 222440/13388 | 24692/  304 |
| BasicZoneManager (all zones+links)     | 229144/13388 | 31396/  304 |
| BasicLinkManager                       | 200920/13220 |  3172/  136 |
| Extended TimeZone (1 zone)             | 206220/13756 |  8472/  672 |
| Extended TimeZone (2 zones)            | 206508/14308 |  8760/ 1224 |
| ExtendedZoneManager (1 zone)           | 207128/13764 |  9380/  680 |
| ExtendedZoneManager (all zones)        | 236940/13764 | 39192/  680 |
| ExtendedZoneManager (all zones+links)  | 244508/13764 | 46760/  680 |
| ExtendedLinkManager                    | 201112/13220 |  3364/  136 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Teensyduino 1.55

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10184/ 4152 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10512/ 4168 |   328/   16 |
| ZonedDateTime                          |  10572/ 4180 |   388/   28 |
| Manual ZoneManager                     |  10308/ 4160 |   124/    8 |
| Basic TimeZone (1 zone)                |  19396/ 4324 |  9212/  172 |
| Basic TimeZone (2 zones)               |  20188/ 4488 | 10004/  336 |
| BasicZoneManager (1 zone)              |  20972/ 4340 | 10788/  188 |
| BasicZoneManager (all zones)           |  38304/ 4340 | 28120/  188 |
| BasicZoneManager (all zones+links)     |  45008/ 4340 | 34824/  188 |
| BasicLinkManager                       |  12048/ 4164 |  1864/   12 |
| Extended TimeZone (1 zone)             |  23216/ 4712 | 13032/  560 |
| Extended TimeZone (2 zones)            |  24008/ 5264 | 13824/ 1112 |
| ExtendedZoneManager (1 zone)           |  24792/ 4716 | 14608/  564 |
| ExtendedZoneManager (all zones)        |  54600/ 4716 | 44416/  564 |
| ExtendedZoneManager (all zones+links)  |  62168/ 4716 | 51984/  564 |
| ExtendedLinkManager                    |  12240/ 4164 |  2056/   12 |
+---------------------------------------------------------------------+

```

